%Main Script to Run Differential Identifiability & CPM
clear; clc; close all;
addpath(genpath('functions'))
addpath(genpath('BrainNetViewer_20181219'))
%% Load the data 
load('DataToRun_NIsymposium.mat');
load('yeo_RSN7_v1_5.mat');

%% Define Upper triangular mask 
numRegions = length(yeoROIs);
utri_mask = triu(true(numRegions,numRegions),1);
diag_mask = logical(eye(numRegions));
figure;spy(utri_mask);
ylabel('Brain Regions');
axis square;title('Upper Triangular Mask');

%% Let's practice going from FC vectorized to FC matrix
FCtest_vectorized = FCvectorized(:,14); %FC data for subject #14
crange = [-0.7 0.7];

%Step1 - Make Empty Connectome of size numRegions x numRegions 
FCtest_matrix = zeros(numRegions); 
h = figure('Units','normalized','Position',[0 0 1 1]);
subplot(1,4,1); imagesc(FCtest_matrix,crange); 
axis square;title('Step1');colorbar;
set(gca,'xtick',[]);set(gca,'ytick',[],'FontSize', 24,'LineWidth',2);

%Step2 - Fill in Upper triangular with values from FCvectorized
FCtest_matrix(utri_mask) = FCtest_vectorized; 
subplot(1,4,2); imagesc(FCtest_matrix,crange); 
axis square;title('Step2');colorbar;
set(gca,'xtick',[]);set(gca,'ytick',[],'FontSize', 24,'LineWidth',2);

%Step3 - Make Symmetric square FC by adding transpose of FC matrix, Make
%Diagonal 1
FCtest_matrix = FCtest_matrix + FCtest_matrix'; 
FCtest_matrix(diag_mask) = 1;
subplot(1,4,3); imagesc(FCtest_matrix,crange); 
axis square;title('Step3');colorbar;
set(gca,'xtick',[]);set(gca,'ytick',[],'FontSize', 24,'LineWidth',2);

%Step4 - Visualize FC by ordering nodes according to RSN network memembership
subplot(1,4,4); imagesc(FCtest_matrix(yeoOrder,yeoOrder),crange); 
axis square;title('Step4');colorbar;
set(gca,'xtick',[]);set(gca,'ytick',[],'FontSize', 24,'LineWidth',2);

%% Configs for Idiff and CPM
%Idiff
configs.max_numPCs = size(FCvectorized,2); %Max number of PCs for Idiff

%CPM
configs.n_bootstraps = 100; %Number of bootstrap iterations to estimate outcome-FC correlations for CPM
configs.ub = 99; %Percentile threshold for positive mask
configs.lb = 1; %Percentile threshold for negative mask
configs.alpha_RSN = 0.01; %significant level for Binomial testing (over represented RSNs)

%define indices for RestA vs. RestB FCs 
%for our [Subj1_A Subj1_B Subj2_A Subj2_B ...]
restA = 1:2:configs.max_numPCs; %odd indices 
restB = 2:2:configs.max_numPCs; %even indices 

%% Run Differential Identifiability (Do I look like myself more than I look like others?)
%I matrix - correlation matrix of FC_restA with FC_restB (Square non symmetric)
%Iself - average of main diagonal (FC_restA~FC_restB same subjects)
%Idiff = average of off diagona elements (FC_restA~FC_restB different subjects)
disp('Running Differential Identifiability');
[I,COEFFS_TestRetest,SCORE_TestRetest,m_star] = f_PCA_identifiability(FCvectorized,restA,restB,configs);
comps = 1:configs.max_numPCs;

% Figure1: Visualize Idiff Results
h = figure('Units','normalized','Position',[0 0 1 1]);
subplot(1,3,1);
set(findobj(gca,'type','line'),'linew',2);
plot(comps,I.diff,comps,100*I.self,comps,100*I.others,'LineWidth',4); hold on;
plot(comps,I.R2*100,'Color',[0.7 0.7 0.7],'LineWidth',4);axis square;
xlabel('# PCs');%ylabel('%');
set(gca,'FontSize',24);
hold on; line([m_star m_star], [0 110],'LineStyle','--','LineWidth',2); text(82, 30,sprintf('%d PCs',m_star),'FontSize',24);hold off;
ylim([0 110]);xlim([1 configs.max_numPCs]);
legend({'$$I_{diff}$$','$$I_{self}$$','$$I_{others}$$','$$R^2$$'},'Interpreter','Latex',...
 'Location','Southwest','FontSize',24);
 
subplot(1,3,2);
imagesc(I.I_matrix(:,:,1),[0 1]);axis square;colorbar;
set(gca,'XTickLabel',[],'YTickLabel',[]);
set(gca,'FontSize', 24);title('ICC-orig');
ylabel('Subjects - RestA');xlabel('Subjects - RestB');
title(sprintf('I - Orig \n $$I_{diff}$$ = %0.2f',I.diff(end)),'Interpreter','Latex');

subplot(1,3,3);
imagesc(I.I_matrix(:,:,2),[0 1]);axis square;colorbar;
set(gca,'XTickLabel',[],'YTickLabel',[]);
set(gca,'FontSize', 24);set(gca,'Linewidth',2');title('ICC-orig');
title(sprintf('I - Optimal \n $$I_{diff}$$ = %0.2f',I.diff(m_star)),'Interpreter','Latex');

%% Assess effect of Idiff at the Edge level using ICC
comps = [configs.max_numPCs m_star]; 
numSubj = size(neurocog_tbl,1);
load('ICC_results.mat');
crange = [0 1];
h = figure('Units','normalized','Position',[0 0 1 1]);
suptitle('Edgewise - Test-Retest ICC');

%preallocate memory
%icc_subj = nan(size(FCvectorized,1),length(comps));
for comp = 1:length(comps)
    disp(sprintf('PC # %d',comps(comp)));
    FCvectorized_temp = f_reconstruct_connectomes(FCvectorized,COEFFS_TestRetest,SCORE_TestRetest,1:comps(comp));
    %icc_temp = f_evaluate_subj_icc(FCvectorized_temp,restA,restB,1:numSubj)'; 
    %icc_subj(:,comp) = icc_temp;
    icc_subj_matrix = f_vec_to_mat(icc_subj(:,comp),numRegions);
    if comp == 1
        t = 'No Idiff';
    else
        t = 'Idiff';
    end
    
    subplot(1,2,comp);
    imagesc(icc_subj_matrix(yeoOrder,yeoOrder),crange); 
    title(t);
    axis square;colorbar;colormap(gca,parula);
    set(gca,'xtick',[]);set(gca,'ytick',[],'FontSize', 18,'LineWidth',2);
    
end

%% CPM -Define Outcomes of Interest for CPM 
outcome = 'Outcome1';
y_use = single(neurocog_tbl.Outcome1);
use_ind_full = ~isnan(y_use);%discard subjects with missing data
leave1out_subjs = find(use_ind_full);

%We will evaluate CPM with original FCs vs. optimally reconstructed FCs
comps = [configs.max_numPCs m_star]; 
%% CPM - Perform Bootstrapped based Edge selection
%Compare edge selection from RestA to RestB for CPM with raw data and Idiff
%before CPM

%Set up figures 
cmap = f_create_rwb_map(2000);
h = figure('Name','EdgeSelection','Units','normalized','Position',[0 0 1 1]);
counter = 1;

%Bootstrap Data & Compute outcome-FC associations for all bootstraps
s = RandStream('mlfg6331_64'); 
bootstrap_subjs = find(use_ind_full);
bootstraps = f_bootstrap_data(s,bootstrap_subjs,configs.n_bootstraps);

%Final outcome-FC association matrix is average correlation across bootstraps
%Pos Mask - configs.ub percentile of FCs, Neg Mask - configs.lb percentile of FCs
for comp = 1:length(comps)
    disp(sprintf('PC # %d',comps(comp)));
    FCvectorized_temp = f_reconstruct_connectomes(FCvectorized,COEFFS_TestRetest,SCORE_TestRetest,1:comps(comp));

    %restA
    FCvectorized_temp_restA = FCvectorized_temp(:,restA);
    [crestA, mpos_restA, mneg_restA] = f_perform_edge_selection(FCvectorized_temp_restA,y_use,bootstraps,configs.lb);
    crestA_matrix = f_vec_to_mat(crestA,numRegions);
    mask_pos_restA(:,comp) = mpos_restA;
    mask_neg_restA(:,comp) = mneg_restA;

    %restB
    FCvectorized_temp_restB = FCvectorized_temp(:,restB);
    [crestB, mpos_restB, mneg_restB] = f_perform_edge_selection(FCvectorized_temp_restB,y_use,bootstraps,configs.lb);
    mask_pos_restB(:,comp) = mpos_restB;
    mask_neg_restB(:,comp) = mneg_restB;

    %Association Matrix Diff
    diff = crestA - crestB;
    diff_matrix = f_vec_to_mat(diff,numRegions);

    if(comp == 1)
        t1 = ('CPM - RestA Association Matrix');
        t2 = ('CPM - (RestA-RestB) Association Matrix');
        t3 = ('CPM - Mask Overlap');
        crange = [prctile(crestA,10),prctile(crestA,90)];
    else
        t1 = ('Idiff + CPM - RestA Association Matrix');
        t2 = ('Idiff + CPM - (RestA - RestB) Association Matrix');
        t3 = ('Idiff + CPM - Mask Overlap');
    end

    %Mask Overlap 
    [overlap,~] = f_evaluate_overlap(mpos_restA,mneg_restA,mpos_restB,mneg_restB);
    overlap_matrix = f_vec_to_mat(overlap,numRegions);
    
    %Visualize Association Matrices      
    subplot(2,3,counter);
    imagesc(crestA_matrix(yeoOrder,yeoOrder),crange); 
    title(t1);
    axis square;colorbar;colormap(gca,parula);
    set(gca,'xtick',[]);set(gca,'ytick',[],'FontSize', 18,'LineWidth',2);  
    
    %Visualize Difference & overlap Matrices      
    subplot(2,3,counter+1);
    imagesc(diff_matrix(yeoOrder,yeoOrder),crange); 
    title(t2);
    axis square;colorbar;colormap(gca,parula);
    set(gca,'xtick',[]);set(gca,'ytick',[],'FontSize', 18,'LineWidth',2);  

    %Visualize Association Matrix        
    subplot(2,3,counter+2);
    imagesc(overlap_matrix(yeoOrder,yeoOrder)); 
    title(t3);
    axis square;colorbar;colormap(gca,cmap);
    set(gca,'xtick',[]);set(gca,'ytick',[],'FontSize', 18,'LineWidth',2);  

    counter = counter+3;
end

%% Estimate Coefficients from RestA Data and RestA (in leave one out fashion)
use_ind_full = ~isnan(y_use);

%Pre Allocate Memory
coeff1_dist_pos_restA = nan(sum(use_ind_full),length(comps));
coeff2_dist_pos_restA = nan(sum(use_ind_full),length(comps));
coeff1_dist_neg_restA = nan(sum(use_ind_full),length(comps));
coeff2_dist_neg_restA = nan(sum(use_ind_full),length(comps));

%Estimate coeff2icients on raw FC and optimally reconstructed FC
for comp = 1:length(comps)

    FCvectorized_temp = f_reconstruct_connectomes(FCvectorized,COEFFS_TestRetest,SCORE_TestRetest,1:comps(comp));
    FCvectorized_temp_restA = FCvectorized_temp(:,restA);
    mask_use_temp_pos = mask_pos_restA(:,comp);
    mask_use_temp_neg = mask_neg_restA(:,comp);

    %RestA Positive Model
    if(sum(mask_use_temp_pos) > 0)
        [coeff1_leaveout,coeff2_leaveout,mse_leaveout, corr_leaveout] = ...
            f_estimate_mdl(FCvectorized_temp_restA,mask_use_temp_pos,y_use);

        coeff1_dist_pos_restA(:,comp) = coeff1_leaveout';
        coeff2_dist_pos_restA(:,comp) = coeff2_leaveout';
    end

    %RestA Negative Model
    if(sum(mask_use_temp_neg) > 0)
        [coeff1_leaveout,coeff2_leaveout,mse_leaveout, corr_leaveout] = ...
            f_estimate_mdl(FCvectorized_temp_restA,mask_use_temp_neg,y_use);

        coeff1_dist_neg_restA(:,comp) = coeff1_leaveout';
        coeff2_dist_neg_restA(:,comp) = coeff2_leaveout';
    end
    
end
%% Estimate Left Out Subjects (compare left out prediction for RestA subjects vs. RestB Subjects)
%Set up figure;
msize = 200;
h = figure('Name',outcome,'Units','normalized','Position',[0 0 1 1]);
suptitle(sprintf('%s \n Accuracy in Prediction of Left Out Subjects',outcome));
counter = 1;

h1 = figure('Name',outcome,'Units','normalized','Position',[0 0 1 1]);
suptitle(sprintf('%s \n Precision in Prediction of Left Out Subjects',outcome));
counter = 1;

%Assess leave one out accuracy for Raw FCs vs. Optimally Reconstructed
%FCs.=
for comp = 1:length(comps)

    FCvectorized_temp = f_reconstruct_connectomes(FCvectorized,COEFFS_TestRetest,SCORE_TestRetest,1:comps(comp));

    %Get masks and coeff2s for this comp
    mask_use_temp_pos = mask_pos_restA(:,comp);
    mask_use_temp_neg = mask_neg_restA(:,comp);
    coeff2_neg = [coeff1_dist_neg_restA(:,comp),coeff2_dist_neg_restA(:,comp)];
    coeff2_pos = [coeff1_dist_pos_restA(:,comp),coeff2_dist_pos_restA(:,comp)];

    %Estimate Leaveout Test with Test Mask and Test Coeffs
    FCvectorized_temp_restA = FCvectorized_temp(:,restA);
    [yhat_leaveout_pos_restA,mse_leaveout_pos_restA,corr_leaveout_pos_restA]  = f_apply_mdl_leaveout(FCvectorized_temp_restA,mask_use_temp_pos,y_use,coeff2_pos);
    [yhat_leaveout_neg_restA,mse_leaveout_neg_restA,corr_leaveout_neg_restA]  = f_apply_mdl_leaveout(FCvectorized_temp_restA,mask_use_temp_neg,y_use,coeff2_neg);
    
    %Estimate Leavout Re-Test with Test Mask and Test Coeffs
    FCvectorized_temp_restB = FCvectorized_temp(:,restB);
    [yhat_leaveout_pos_restB,mse_leaveout_pos_restB,corr_leaveout_pos_restB]  = f_apply_mdl_leaveout(FCvectorized_temp_restB,mask_use_temp_pos,y_use,coeff2_pos);
    [yhat_leaveout_neg_restB,mse_leaveout_neg_restB,corr_leaveout_neg_restB] = f_apply_mdl_leaveout(FCvectorized_temp_restB,mask_use_temp_neg,y_use,coeff2_neg);
    
    %Calc correlation between prediction from RestA and RestB FCs
    corr_leaveout_pos_restA_restB = corr(yhat_leaveout_pos_restA',yhat_leaveout_pos_restB');
    corr_leaveout_neg_restA_restB = corr(yhat_leaveout_neg_restA',yhat_leaveout_neg_restB');
    
    if(comp == 1)
        t1 = 'CPM Positive Model';
        t2 = 'CPM Negative Model';
    else
        t1 = 'Idiff + CPM Positive Model';
        t2 = 'Idiff + CPM Negative Model';
    end
    
    %Visualize Model Accuracy
    %Positive Model 
    figure(h);subplot(2,2,counter);
    scatter(y_use(use_ind_full),yhat_leaveout_pos_restA,msize,'b','filled','MarkerEdgeColor','k');
    hold on;
    scatter(y_use(use_ind_full),yhat_leaveout_pos_restB,msize,'r','filled','MarkerEdgeColor','k'); 
    title(sprintf('%s \n rRestA = %0.2f, rRestB = %0.2f',t1,corr_leaveout_pos_restA,corr_leaveout_pos_restB));
    legend({'RestA','RestB'},'Location','NorthWest');
    set(gca,'FontSize', 18,'LineWidth',2); axis square;
    xlabel('actual outcome');ylabel('estimated outcome');
    hold off;

    %Negative Model
    figure(h);subplot(2,2,counter+1);
    scatter(y_use(use_ind_full),yhat_leaveout_neg_restA,msize,'b','filled','MarkerEdgeColor','k');
    hold on;
    scatter(y_use(use_ind_full),yhat_leaveout_neg_restB,msize,'r','filled','MarkerEdgeColor','k');
    title(sprintf('%s \n rRestA = %0.2f, rRestB = %0.2f',t2,corr_leaveout_neg_restA,corr_leaveout_neg_restB));
    legend({'RestA','RestB'},'Location','NorthWest');
    xlabel('actual outcome');ylabel('estimated outcome');
    set(gca,'FontSize', 18,'LineWidth',2);axis square;
    hold off;
    
    %Visualize Model Specificity
    %Positive Model
    figure(h1);subplot(2,2,counter);
    scatter(yhat_leaveout_pos_restA,yhat_leaveout_pos_restB,msize,'g','filled','MarkerEdgeColor','k');
    title(sprintf('%s \n rRestA = %0.2f',t1,corr_leaveout_pos_restA_restB));
    set(gca,'FontSize', 18,'LineWidth',2); axis square;
    xlabel('estimated outcome restA');ylabel('estimated outcome restB');
    hold off;

    %Negative Model
    figure(h1);subplot(2,2,counter+1);
    scatter(yhat_leaveout_neg_restA,yhat_leaveout_neg_restB,msize,'g','filled','MarkerEdgeColor','k');
    title(sprintf('%s \n rRestA = %0.2f',t1,corr_leaveout_neg_restA_restB));
    set(gca,'FontSize', 18,'LineWidth',2); axis square;
    xlabel('estimated outcome restA');ylabel('estimated outcome restB');
    hold off;

    counter = counter+2;
end

%% Estimate Hyper Parameters

%Positive Model
mask_pos_final = mask_pos_restA(:,2);
coeff2s_final_pos = [mean(coeff1_dist_pos_restA(:,2)), mean(coeff2_dist_pos_restA(:,2))];

%Negative Model
mask_neg_final = mask_neg_restA(:,2);
coeff2s_final_neg = [mean(coeff1_dist_neg_restA(:,2)), mean(coeff2_dist_neg_restA(:,2))];

%% Find Over Represented Networks
mask_str_pos = sprintf('%s_pos_model',outcome);
mask_str_neg = sprintf('%s_neg_model',outcome);
output_dir = 'MaskVisualizations';
[sig_RSNs_over_pos,sig_RSNs_under_pos] = f_evaluate_RSN(mask_pos_final,configs.alpha_RSN,mask_str_pos,output_dir);
[sig_RSNS_over_neg,sig_RSNs_under_neg] = f_evaluate_RSN(mask_neg_final,configs.alpha_RSN,mask_str_neg,output_dir);
